/*
Teletekst.cs - Teletekst Viewer voor .NET Compact Framework

Copyright (C) 2003-2005, Ward van Wanrooij
All Rights Reserved

Meest recente versie beschikbaar op http://www.ward.nu/computer/teletekst
*/

using System;
using System.Net;
using System.Windows.Forms;
using System.Drawing;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Resources;
using System.Reflection;
using System.Collections;

[assembly: System.Reflection.AssemblyVersion("0.0.5.0")]

namespace Ward.Teletekst {
	public class TeletekstPage
	{
		protected int page, subpage, nextpage=0, prevpage=0, nextsubpage=0, prevsubpage=0;
		protected String contents="";
	
	   	public TeletekstPage(int page, int subpage) {
			this.page=page;
			this.subpage=subpage;
		}
	
		public String Contents {
			get { return(contents); }
		}
	
		public int Page {
			get { return(page); }
		}
	
		public int SubPage {
			get { return(subpage); }
		}
	
		public int NextPage {
			get { return(nextpage); }
		}
	
		public int PreviousPage {
			get { return(prevpage); }
		}
	
		public int NextSubPage {
			get { return(nextsubpage); }
		}
	
		public int PreviousSubPage {
			get { return(prevsubpage); }
		}
	
		public Boolean Retrieve() {
			WebResponse res;
			Boolean funcres;
			ResourceManager rm;

			res=null;
			funcres=true;
			contents="";
			nextpage=0;
			prevpage=0;		
			nextsubpage=0;
			prevsubpage=0;		
			if ((page==0) & (subpage==0)) {
				contents="|black| ";
			} else if ((page==999) & (subpage==1)) {
				rm=new ResourceManager("Teletekst", Assembly.GetExecutingAssembly());
				contents=rm.GetString("aboutpage");
			} else {
				try {
					HttpWebRequest req;
					StreamReader sr;
					Char[] read;
					Regex regpage;
					MatchCollection matches;
					String pagecont, url, desc;

					pagecont="";
					if (page>899) {
						url="http://www.ward.nu/cgi-bin/teletekst.cgi?page="+page.ToString("000")+"&subpage="+subpage.ToString("00")+"&version="+Assembly.GetExecutingAssembly().GetName().Version.ToString();
					} else {
						url="http://teletekst.nos.nl/tekst/"+page.ToString("000")+"-"+subpage.ToString("00")+".html";
					}
					int count;
					req=(HttpWebRequest)WebRequest.Create(url);
		  			req.Accept="*/*";
					req.Referer=url;
					req.UserAgent="Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322)";
					req.Headers.Set("Accept-Language", "nl");
					res=req.GetResponse();
					sr=new StreamReader(res.GetResponseStream(), Encoding.GetEncoding("windows-1252"));
					read=new Char[1024];
					count=sr.Read(read, 0, read.Length);
					while (count>0) {
						pagecont+=new String(read, 0, count);
						count=sr.Read(read, 0, read.Length);
		        	        }
		                	res.Close();
	
					regpage=new Regex("<A\\s+HREF=\"(?<page>\\d+)-(?<subpage>\\d+).html\"[^>]+>(?<desc>[&lgt;]+)</", RegexOptions.Singleline | RegexOptions.IgnoreCase);
					matches=regpage.Matches(pagecont);
					foreach(Match mtch in matches) {
						desc=mtch.Groups["desc"].Value;
						if (desc.Equals("&lt;")) {
							prevpage=Convert.ToInt32(mtch.Groups["page"].Value);
						} else if (desc.Equals("&gt;")) {
							nextpage=Convert.ToInt32(mtch.Groups["page"].Value);
						} else if (desc.Equals("&lt;&lt;")) {
							prevsubpage=Convert.ToInt32(mtch.Groups["subpage"].Value);
						} else if (desc.Equals("&gt;&gt;")) {
							nextsubpage=Convert.ToInt32(mtch.Groups["subpage"].Value);
						}
					}
					regpage=new Regex("<PRE>(?<tt>.+)</PRE>", RegexOptions.Singleline | RegexOptions.IgnoreCase);
					matches=regpage.Matches(pagecont); 
					if (matches.Count==1) {
						pagecont=matches[0].Groups["tt"].ToString();
					} else {
						throw(new InvalidOperationException("Invalid page contents."));
					}
					//pagecont=Regex.Replace(pagecont, @"^.*?[\n\r]+", "", RegexOptions.Singleline);
					pagecont=Regex.Replace(pagecont, @"(.+)[\n\r]+[^\n\r]+[\n\r]+$", "$1", RegexOptions.Singleline);
					pagecont=Regex.Replace(pagecont, @"&\w*;", "'", RegexOptions.IgnoreCase);
					pagecont=Regex.Replace(pagecont, @"</?A[^>]*?>", "", RegexOptions.IgnoreCase);
					pagecont=Regex.Replace(pagecont, @"<FONT color=(\w+)>", "|$1|", RegexOptions.IgnoreCase);
					pagecont=Regex.Replace(pagecont, @"</FONT>", "", RegexOptions.IgnoreCase);
					pagecont=Regex.Replace(pagecont, @"(^|[\n\r]+)([\|\w]+) ", "$2", RegexOptions.Singleline);
					contents=pagecont;
				} catch {
					rm=new ResourceManager("Teletekst", Assembly.GetExecutingAssembly());
					contents=rm.GetString("errorpage");					
					funcres=false;
			        } finally {
		        		if (res!=null ) {
						res.Close();
					}
				}
		        }
			return(funcres);
		}
	}
	
	public class TeletekstNumber: Form
	{
		private const int SCREEN_WIDTH=240;
		private const int SCREEN_HEIGHT=294;
		private Button[] btns;
		private Button btnclear, btnok;
		private Label lblpagenumber;
		private String pagenumber;
		private int pagekey;
		private Boolean inputkey;
	
		public TeletekstNumber()
		{
			TeletekstNumber_Load();
		}
	
		public String PageNumber
		{
			get { return(pagenumber); }
		}	
		
		private void TeletekstNumber_Load()
		{
			pagenumber="";
			pagekey=0;
			inputkey=false;
			ClientSize=new Size(SCREEN_WIDTH, SCREEN_HEIGHT);
			Text="Teletekst Pagina";
			MaximizeBox=false;
			MinimizeBox=false;
			lblpagenumber=new Label();
			lblpagenumber.Font=new Font("Tahoma", 28, FontStyle.Bold);
			lblpagenumber.Location=new Point(5, 0);
			lblpagenumber.Text="";
			lblpagenumber.Width=240;
			lblpagenumber.Height=40;
			Controls.Add(lblpagenumber);
			btns=new Button[12];
			for (int i=0; i<12; i++) {
				btns[i]=new Button();
				btns[i].Font=new Font("Tahoma", 28, FontStyle.Bold);
				btns[i].Location=new Point(10+(i%3)*75, 60+Convert.ToInt32(Decimal.Floor(i/3))*60);
				btns[i].Size=new Size(65, 50);
				btns[i].Text="&"+Convert.ToString(i+1);
				btns[i].Click+=new EventHandler(btns_Click);
				btns[i].KeyUp+=new KeyEventHandler(TeletekstNumber_KeyUp);
				Controls.Add(btns[i]);
			}
			btns[9].Text="&Cl";
			btns[9].Font=new Font("Tahoma", 20, FontStyle.Bold);
			btns[10].Text="&0";
			btns[11].Text="&OK";
			btns[11].Font=new Font("Tahoma", 20, FontStyle.Bold);
			btns[11].Enabled=false;
			btnclear=btns[9];
			btnok=btns[11];
			KeyUp+=new KeyEventHandler(TeletekstNumber_KeyUp);
		}

		private void AddNumber(Char number)
		{
			if (pagenumber.Length==3) {
				pagenumber+="-"+number;
			} else if (pagenumber.Length!=6) {
				pagenumber+=number;
			}
			inputkey=false;
		}

		private void DelNumber()
		{
			if (pagenumber.Length==0) {
				this.DialogResult=DialogResult.Cancel;
			} else if (pagenumber.Length==5) {
				pagenumber=pagenumber.Substring(0, 3);
			} else {			
				pagenumber=pagenumber.Substring(0, pagenumber.Length-1);			
			}
		}

		private void UpdateNumber()
		{
			if (inputkey) { 
				lblpagenumber.Text=pagenumber+((pagenumber.Length==3)?"-&":"&")+Convert.ToString(pagekey);
			} else {
				lblpagenumber.Text=pagenumber;
			}
			btnok.Enabled=(pagenumber.Length>2);
		}

		private void ConfirmNumber()
		{
			if (btnok.Enabled) {
				this.DialogResult=DialogResult.OK;
			} else {
				this.DialogResult=DialogResult.Cancel;
			}	
		}
		
		private void TeletekstNumber_KeyUp(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			if ((e.KeyCode==Keys.Up) | (e.KeyCode==Keys.Down)) {
				if (inputkey) {
					pagekey=Math.Abs(pagekey+10+((e.KeyCode==Keys.Up)?1:-1))%10;
				} else {
					pagekey=(e.KeyCode==Keys.Up)?0:9;
				}
				inputkey=true;
			} else if ((e.KeyCode==Keys.Right) | ((e.KeyCode==Keys.Return) & ((sender==btnok) | (sender==this)))) {
				if (inputkey) {
					AddNumber(Convert.ToChar(Convert.ToString(pagekey)));
				} else {
					ConfirmNumber();
				}
			} else if (e.KeyCode==Keys.Left) {
				if (inputkey) {
					inputkey=false;
				} else {
					DelNumber();
				}
			} else if (e.KeyCode==Keys.Back) {
				DelNumber();
			}
			if ((e.KeyCode==Keys.Left) | (e.KeyCode==Keys.Up) | (e.KeyCode==Keys.Down) | (e.KeyCode==Keys.Right) | (e.KeyCode==Keys.Return) | (e.KeyCode==Keys.Back)) {
				e.Handled=true;
				UpdateNumber();
			}
		}
	
		private void btns_Click(object sender, EventArgs e)
		{
			Button btn;

			btn=(Button)sender;
			if(btn.Equals(btnok)) {
				ConfirmNumber();
			} else if(btn.Equals(btnclear)) {
				DelNumber();
			} else {
				if (btn==btns[10]) {
					AddNumber('0');
				} else {
					for (int i=0; i<9; i++) {
						if (btns[i].Equals(btn)) {
							AddNumber(Convert.ToChar(Convert.ToString(i+1)));
							break;
						}
					}
				}
			}
			UpdateNumber();
		}
	}
	
	public class TeletekstViewer: Form
	{
		private const int SCREEN_WIDTH=240;
		private const int SCREEN_HEIGHT=294;
		private const int PAGE_ROWS=24;
		private const int PAGE_COLS=39;
		private const int CHAR_WIDTH=6;
		private const int CHAR_HEIGHT=11;
		private const int HISTORY_MAX=10;
		private const int FAV_MAX=10;
		private String FAV_FILE="nosttnet.txt";
		private PictureBox telebox;
		private Bitmap cachedbmp;
		private TeletekstPage curpage;
		private ToolBarButton btngopage, btnprevpage, btnprevsubpage, btnhomepage, btnnextsubpage, btnnextpage, btnhistory, btnfav;
		private ToolBar toolbar;
		private ContextMenu historymenu, favmenu;
	
		public TeletekstViewer()
		{
			if (Environment.OSVersion.Platform==PlatformID.WinCE ) {
				FAV_FILE=@"\Windows\"+FAV_FILE;
			}
			Closed+=new EventHandler(TeletekstViewer_Closed);
			TeletekstViewer_Load();
		}
	
		private void TeletekstViewer_Load()
		{
			ResourceManager rm;
			ImageList tbil;
			ToolBarButton[] tbtns;

			rm=new ResourceManager("Teletekst", Assembly.GetExecutingAssembly());
			ClientSize=new Size(SCREEN_WIDTH, SCREEN_HEIGHT);
			MinimizeBox=(Environment.OSVersion.Platform!=PlatformID.WinCE);
			MaximizeBox=false;			
			Text="Teletekst Viewer";
			Icon=(System.Drawing.Icon)rm.GetObject("teletekst");
			tbil=new ImageList();
			tbil.Images.Add((System.Drawing.Icon)rm.GetObject("select"));
			tbil.Images.Add((System.Drawing.Icon)rm.GetObject("teletekst"));
			tbil.Images.Add((System.Drawing.Icon)rm.GetObject("left"));
			tbil.Images.Add((System.Drawing.Icon)rm.GetObject("up"));
			tbil.Images.Add((System.Drawing.Icon)rm.GetObject("home"));
			tbil.Images.Add((System.Drawing.Icon)rm.GetObject("down"));
			tbil.Images.Add((System.Drawing.Icon)rm.GetObject("right"));
			tbil.Images.Add((System.Drawing.Icon)rm.GetObject("teletekst"));
			tbil.Images.Add((System.Drawing.Icon)rm.GetObject("history"));
			tbil.Images.Add((System.Drawing.Icon)rm.GetObject("favorite"));
			historymenu=new ContextMenu();
			favmenu=new ContextMenu();
			favmenu.Popup+=new EventHandler(favmenu_Popup);
			toolbar=new ToolBar();
			toolbar.ImageList=tbil;
			toolbar.ButtonClick+=new ToolBarButtonClickEventHandler(btns_ButtonClick);
			tbtns=new ToolBarButton[10];
			for (int i=0; i<tbtns.Length; i++) {
				tbtns[i]=new ToolBarButton();
				tbtns[i].Style=ToolBarButtonStyle.PushButton;
				tbtns[i].ImageIndex=i;
     				toolbar.Buttons.Add(tbtns[i]);
			}
			tbtns[1].Style=ToolBarButtonStyle.Separator;
			tbtns[1].ImageIndex=-1;
			tbtns[7].Style=ToolBarButtonStyle.Separator;
			tbtns[7].ImageIndex=-1;
			tbtns[8].Style=ToolBarButtonStyle.DropDownButton;
			tbtns[8].DropDownMenu=historymenu;
			tbtns[8].Enabled=false;
			tbtns[9].Style=ToolBarButtonStyle.DropDownButton;
			tbtns[9].DropDownMenu=favmenu;
			btngopage=tbtns[0];
			btnprevpage=tbtns[2];
			btnprevsubpage=tbtns[3];
			btnhomepage=tbtns[4];
			btnnextsubpage=tbtns[5];
			btnnextpage=tbtns[6];
			btnhistory=tbtns[8];
			btnfav=tbtns[9];
			Controls.Add(toolbar);
			LoadFavourites(rm.GetString("favorites"));
			telebox=new PictureBox();
			telebox.Left=0;
			telebox.Width=SCREEN_WIDTH;
			if (Environment.OSVersion.Platform==PlatformID.WinCE ) {
				telebox.Top=0;
			} else {
				telebox.Top=26;
			}
			telebox.Height=PAGE_ROWS*CHAR_HEIGHT+5;
			telebox.BackColor=Color.Black;
			telebox.Paint+=new PaintEventHandler(telebox_Paint);
			telebox.Click+=new EventHandler(telebox_Click);
			curpage=null;
			LoadPage(999, 1);
			Controls.Add(telebox);	
			KeyUp+=new KeyEventHandler(TeletekstViewer_KeyUp);
			telebox.KeyUp+=new KeyEventHandler(TeletekstViewer_KeyUp);
			toolbar.KeyUp+=new KeyEventHandler(TeletekstViewer_KeyUp);
		}


		private void TeletekstViewer_KeyUp(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			ToolBarButton btn=null;
			MenuItem mi;

			switch (e.KeyCode) {
				case Keys.Left: btn=btngopage; break;
				case Keys.Up: btn=btnprevsubpage; break;
				case Keys.Down: btn=btnnextsubpage; break;
				case Keys.Right: btn=btnfav; break;
				case Keys.G: btn=btngopage; break;
				case Keys.F: btn=btnfav; break;
				//case Keys.Return: btn=btngopage; break;
			}
			if (btn!=null) {
				if (btn==btnfav) {
					mi=new MenuItem();
					mi.Text="Sluiten";
					favmenu.MenuItems.Add(mi);
					btns_ButtonClick(null, new ToolBarButtonClickEventArgs(btn));
					favmenu.MenuItems.Remove(mi);
				} else {	
					btns_ButtonClick(null, new ToolBarButtonClickEventArgs(btn));
				}
				e.Handled=true;
			}
		}

		private void TeletekstViewer_Closed(object sender, EventArgs e)
		{
			SaveFavourites();
		}
	
		private void telebox_Paint(object sender, PaintEventArgs e)
		{
			if (cachedbmp!=null) {
				e.Graphics.DrawImage(cachedbmp, e.ClipRectangle, e.ClipRectangle, GraphicsUnit.Pixel);
			} else {
				Graphics g;
				Font drawFont;
				SolidBrush drawBrush;
				String newchar, newcolor, pagstr;
				Boolean incolor;
				int i, j;
				
				cachedbmp=new Bitmap(telebox.Width, telebox.Height);
				g=Graphics.FromImage(cachedbmp);
				drawFont=new Font("Courier New", 9, FontStyle.Regular);
				drawBrush=new SolidBrush(Color.Black);
				pagstr=curpage.Contents;
				newcolor="";
				incolor=false;
				j=0;
	
				e.Graphics.FillRectangle(drawBrush, 0, 0, telebox.Width, telebox.Height);
				g.FillRectangle(drawBrush, 0, 0, telebox.Width, telebox.Height);
				for (i=0; i<pagstr.Length; i++) {
					newchar=pagstr[i].ToString();						
					if (newchar.Equals("|")) {
						if (incolor) {
							drawBrush=new SolidBrush(ColorFromName(newcolor));
							newcolor="";
							incolor=false;
						} else {
							incolor=true;
						}
					} else {
						if (incolor) {
							newcolor+=newchar;
						} else {
							if (newchar.Equals("*")) {
								g.FillRectangle(drawBrush, new Rectangle((j % PAGE_COLS)*CHAR_WIDTH+3, Convert.ToInt32(Decimal.Floor(j / PAGE_COLS)*CHAR_HEIGHT+3), CHAR_WIDTH, CHAR_HEIGHT));
							} else {
								g.DrawString(newchar, drawFont, drawBrush, new Rectangle((j % PAGE_COLS)*CHAR_WIDTH+3, Convert.ToInt32(Decimal.Floor(j / PAGE_COLS)*CHAR_HEIGHT), CHAR_WIDTH+3, CHAR_HEIGHT+3));
							}
							j++;
						}
					}
				}
				e.Graphics.DrawImage(cachedbmp, 0, 0);
			}
		}
		
		private void telebox_Click(object sender, EventArgs e) {
			Point pnt;
			Regex r;
			Match m;
			String pagstr, newpage, newchar;
			Boolean incolor;
			int i, j, charnr;

			pnt=telebox.PointToClient(Control.MousePosition);
			charnr=Convert.ToInt32(Decimal.Floor(pnt.X/CHAR_WIDTH)+Decimal.Floor(pnt.Y/CHAR_HEIGHT)*PAGE_COLS);
			pagstr=curpage.Contents;
			incolor=false;
			newpage="";
			j=0;
			for (i=0; i<pagstr.Length; i++) {
				newchar=pagstr[i].ToString();					
				if (newchar.Equals("|")) {
					incolor=!incolor;
				} else if (!incolor) {
					j++;
					if ((j>charnr-4) & (j<charnr+4)) {
						newpage+=newchar;
					}
				}
			}
			r=new Regex(@"\D(\d{3})\D"); 
			m=r.Match(newpage); 
			if (m.Success) {
				Invalidate();
				LoadPage(Convert.ToInt32(m.Groups[1].Value), 1);
			}
		}

		private void pagemenuitem_Click(object sender, EventArgs e) {
			MenuItem mi;
			int newpage, newsubpage;

			mi=(MenuItem)sender;
			newpage=0;
			newsubpage=0;
			if (ParsePage(mi.Text, ref newpage, ref newsubpage)) {
				LoadPage(newpage, newsubpage);
			}
		}

		private void favmenu_Popup(Object sender, System.EventArgs e) {
			//Doesn't get fired under CE
			if (MenuItemCollectionGetIndexByText(favmenu.MenuItems, Convert.ToString(curpage.Page)+"-"+Convert.ToString(curpage.SubPage))==-1) {
				favmenu.MenuItems[0].Text="Toevoegen";
			} else {
				favmenu.MenuItems[0].Text="Verwijderen";
			}			
		}
		
		private void favmenuaction_Click(object sender, EventArgs e) {
			MenuItem mi;

			mi=(MenuItem)sender;
			if (mi.Text.Equals("Toevoegen")) {
				mi=new MenuItem();
				mi.Text=Convert.ToString(curpage.Page)+"-"+Convert.ToString(curpage.SubPage);
				mi.Click+=new System.EventHandler(pagemenuitem_Click);
				favmenu.MenuItems.Add(mi);
			} else if (mi.Text.Equals("Verwijderen")) {
				favmenu.MenuItems.RemoveAt(MenuItemCollectionGetIndexByText(favmenu.MenuItems, Convert.ToString(curpage.Page)+"-"+Convert.ToString(curpage.SubPage)));
			}
			if (Environment.OSVersion.Platform==PlatformID.WinCE ) {
				favmenu_Popup(null, null);
			}
		}

		private void btns_ButtonClick(object sender, ToolBarButtonClickEventArgs e) {
			ToolBarButton btn;
			int newpage, newsubpage;

			btn=e.Button;
			if (!(btn.Enabled)) { return; }
			newpage=0;
			newsubpage=0;
			if ((btn.Equals(btnprevpage)) | (btn.Equals(btnnextpage))) {
				newsubpage=1;
			} else if ((btn.Equals(btnprevsubpage)) | (btn.Equals(btnnextsubpage))) {
				newpage=curpage.Page;
			}
			if (btn.Equals(btnprevpage)) {
				newpage=curpage.PreviousPage;
			} else if (btn.Equals(btnnextpage)) {
				newpage=curpage.NextPage;
			} else if (btn.Equals(btnnextsubpage)) {
				newsubpage=curpage.NextSubPage;
			} else if (btn.Equals(btnprevsubpage)) {
				newsubpage=curpage.PreviousSubPage;
			} else if (btn.Equals(btnhomepage)) {
				newpage=100;
				newsubpage=1;
			} else if (btn.Equals(btngopage)) {
				TeletekstNumber tn;

				tn=new TeletekstNumber();
				if (tn.ShowDialog()==DialogResult.OK) {
					ParsePage(tn.PageNumber, ref newpage, ref newsubpage);
				}					
				tn.Dispose();
			} else if (btn.Equals(btnhistory)) {
				if (historymenu.MenuItems.Count>0) {
					ParsePage(historymenu.MenuItems[historymenu.MenuItems.Count-1].Text, ref newpage, ref newsubpage);
				}
			} else if (btn.Equals(btnfav)) {
				if (Environment.OSVersion.Platform==PlatformID.WinCE) {
					favmenu.Show(telebox, new Point(191, telebox.Height+14-favmenu.MenuItems.Count*18));
				} else {
					favmenu.Show(toolbar, new Point(191, 24));
				}
			}
			if ((newpage!=0) & (newsubpage!=0)) {
				LoadPage(newpage, newsubpage);		
			}
		}

		private Boolean ParsePage(String page, ref int newpage, ref int newsubpage) {
			Regex r;
			Match m;

			r=new Regex(@"(\d{3})(-(\d+))?"); 
			m=r.Match(page); 
			if (m.Success) {
				newpage=Convert.ToInt32(m.Groups[1].Value);
				newsubpage=(m.Groups[3].Value.Length==0) ? 1 : Convert.ToInt32(m.Groups[3].Value);
				return(true);
			} else {
				return(false);
			}
		}
	
		private void LoadPage(int page, int subpage) {
			Cursor OldCursor;
			MenuItem mi;

			OldCursor=Cursor.Current;
			Cursor.Current=Cursors.WaitCursor;
			if (curpage!=null) {
				mi=new MenuItem();
				mi.Text=Convert.ToString(curpage.Page)+"-"+Convert.ToString(curpage.SubPage);
				mi.Click+=new System.EventHandler(pagemenuitem_Click);
				historymenu.MenuItems.Add(mi);
				if (historymenu.MenuItems.Count>HISTORY_MAX) {
					historymenu.MenuItems.RemoveAt(0);
				}
				btnhistory.Enabled=true;
			}
			curpage=new TeletekstPage(page, subpage);
			curpage.Retrieve();					
			btnprevpage.Enabled=(curpage.PreviousPage!=0);
			btnprevsubpage.Enabled=(curpage.PreviousSubPage!=0);
			btnnextsubpage.Enabled=(curpage.NextSubPage!=0);
			btnnextpage.Enabled=(curpage.NextPage!=0);
			if (Environment.OSVersion.Platform==PlatformID.WinCE ) {
				favmenu_Popup(null, null);
			}
			cachedbmp=null;
			toolbar.Invalidate();
			telebox.Invalidate();
			Cursor.Current=OldCursor;
		}

		private MenuItem ConvertMenu(String menudesc, String menutitle) {
			int i;
			Char newchar;
			MenuItem mi, parent;
			
			i=0;
			parent=new MenuItem();
			parent.Text=menutitle;			
			menutitle="";
			while (i<menudesc.Length) {
				newchar=Convert.ToChar(menudesc[i].ToString());					
				if ((newchar.Equals('|')) | (newchar.Equals('>')) | (newchar.Equals('<'))) {
					if (newchar.Equals('>')) {
						mi=ConvertMenu(menudesc.Substring(i+1, menudesc.Substring(i).IndexOf('<')), menutitle);
						i+=menudesc.Substring(i).IndexOf('<');
					} else {
						mi=new MenuItem();
						mi.Text=menutitle;
						mi.Click+=new System.EventHandler(pagemenuitem_Click);
					}						
					parent.MenuItems.Add(mi);
					menutitle="";
				} else {
					menutitle+=newchar;
				}
				i++;
			}
			return(parent);
		}

		private void LoadFavourites(String favdefault) {
			MenuItem mi;
			StreamReader sr;
		        String input;

			mi=new MenuItem();
			mi.Text="action";
			mi.Click+=new System.EventHandler(favmenuaction_Click);
			favmenu.MenuItems.Add(mi);
			mi=new MenuItem();
			mi.Text="-";
			favmenu.MenuItems.Add(mi);
			favmenu.MenuItems.Add(ConvertMenu(favdefault, "Standaard"));
			mi=new MenuItem();
			mi.Text="-";
			favmenu.MenuItems.Add(mi);

			if (File.Exists(FAV_FILE)) {
        			sr=File.OpenText(FAV_FILE);
				while (((input=sr.ReadLine())!=null) & (favmenu.MenuItems.Count<FAV_MAX+2)) {
					mi=new MenuItem();
					mi.Text=input;
					mi.Click+=new System.EventHandler(pagemenuitem_Click);
					favmenu.MenuItems.Add(mi);
				}
			        sr.Close();
			}
		}

		private void SaveFavourites() {
			StreamWriter sw;
			
       			sw=new StreamWriter(FAV_FILE);
			for (int i=4; i<favmenu.MenuItems.Count; i++) {
				sw.WriteLine(favmenu.MenuItems[i].Text);
			}
		        sw.Close();
		}
			
		private int MenuItemCollectionGetIndexByText(Menu.MenuItemCollection mic, String txt)
		{
			for (int i=0; i<mic.Count; i++) {
				if (mic[i].Text.Equals(txt)) {
					return(i);
				}
			}
			return(-1);
		}
		
		private Color ColorFromName(String FromName) 
		{
			//Color.FromName is not supported under .NET CF
			if (FromName.Equals("black")) {
				return(Color.Black);
			} else if (FromName.Equals("maroon")) {
				return(Color.Maroon);
			} else if (FromName.Equals("green")) {
				return(Color.Green);
			} else if (FromName.Equals("olive")) {
				return(Color.Olive);
			} else if (FromName.Equals("navy")) {
				return(Color.Navy);
			} else if (FromName.Equals("purple")) {
				return(Color.Purple);
			} else if (FromName.Equals("teal")) {
				return(Color.Teal);
			} else if (FromName.Equals("silver")) {
				return(Color.Silver);
			} else if (FromName.Equals("red")) {
				return(Color.Red);
			} else if (FromName.Equals("lime")) {
				return(Color.Lime);
			} else if (FromName.Equals("yellow")) {
				return(Color.Yellow);
			} else if (FromName.Equals("gray")) {
				return(Color.Gray);
			} else if (FromName.Equals("blue")) {
				return(Color.Blue);
			} else if (FromName.Equals("fuchsia")) {
				return(Color.Fuchsia);
			} else if (FromName.Equals("aqua")) {
				return(Color.Aqua);
			} else if (FromName.Equals("white")) {
				return(Color.White);
			} else {
				return(Color.White);
			}
		}
		
		static void Main()
		{
			Application.Run(new TeletekstViewer());
		}
	}
}
